//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Unit1.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"

class TArticle : public TObject {
  private:
    String FName;
    float FPrice;
    int FStock;
  public:
    __property String Name = { read = FName, write = FName };
    __property float Price = { read = FPrice, write = FPrice };
    __property int Stock = { read = FStock, write = FStock };
};

String DBMode; // "B" for Browse, "A" for Add, "E" for Edit

TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
  : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormCreate(TObject *Sender)
{
  Articles = new TStringList;
  Articles->CaseSensitive = false;
  Articles->Sorted = true;
  // the stringlist's objects must be automatically freed upon...
  // ...destroying the stringlist
  Articles->OwnsObjects = true;

  stStatus->Caption = "";
  edFind->Text = "";
  DBMode = "B";
  UpdateUI("");
  Grid->ColWidths[0] = 60;
  Grid->ColWidths[1] = 120;
  Grid->ColWidths[2] = 80;
  Grid->ColWidths[3] = 60;
  Grid->Cells[0][0] = "ID";
  Grid->Cells[1][0] = "Name";
  Grid->Cells[2][0] = "Price";
  Grid->Cells[3][0] = "Stock";
}
//----------------------------------------------------------------------------
void TForm1::UpdateUI(String Stat)
{
  stStatus->Caption = Stat;
  Panel1->Visible = ((DBMode == "A") || (DBMode == "E"));
  // disable buttons when Panel1 is visible
  btnAdd->Enabled = !(Panel1->Visible);
  btnEdit->Enabled = !(Panel1->Visible);
  btnDelete->Enabled = !(Panel1->Visible);
  btnFindID->Enabled = !(Panel1->Visible);
  btnFindName->Enabled = !(Panel1->Visible);

  if (DBMode == "A") {  // clear edit boxes
    edID->Text = "";
    edName->Text = "";
    edPrice->Text = "";
    edStock->Text = "";
  }
  if ((DBMode == "A") || (DBMode == "E"))
    edID->SetFocus();
}
//----------------------------------------------------------------------------
void TForm1::UpdateGrid()
{
  int index;
  Grid->RowCount = Articles->Count + 1;
  for (index = 0; index < Articles->Count; index++)
  {
    TArticle *Art = static_cast<TArticle*>(Articles->Objects[index]);
    Grid->Cells[0][index + 1] = Articles->Strings[index];
    Grid->Cells[1][index + 1] = Art->Name;
    Grid->Cells[2][index + 1] = FormatFloat("0.00", Art->Price);
    Grid->Cells[3][index + 1] = IntToStr(Art->Stock);
  }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::btnAddClick(TObject *Sender)
{
  DBMode = "A";
  UpdateUI(" Adding article ");
}
//---------------------------------------------------------------------------
void __fastcall TForm1::btnEditClick(TObject *Sender)
{
  int index;
  if (Grid->Row < 1)
    stStatus->Caption = " Select article to be edited ";
  else {
    DBMode = "E";
    UpdateUI(" Editing article ");
    index = Grid->Row - 1;
    TArticle *Art = static_cast<TArticle*>(Articles->Objects[index]);
    edID->Text = Articles->Strings[index];
    edName->Text = Art->Name;
    edPrice->Text = FormatFloat("0.00", Art->Price);
    edStock->Text = IntToStr(Art->Stock);
  }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::btnDeleteClick(TObject *Sender)
{
  int index;
  if (Grid->Row < 1)
    stStatus->Caption = " Select article to be deleted ";
  else {
    if (MessageDlg("Really delete this article?",
                   mtConfirmation, mbYesNoCancel, 0) == mrYes) {
      Articles->Delete(index);
      UpdateGrid();
      stStatus->Caption = " Article was deleted ";
    }
    else
      stStatus->Caption = " Article was not deleted ";
  }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::btnOKClick(TObject *Sender)
{
  int index;
  String ID;
  ID = edID->Text;
  TArticle *Art = new TArticle;
  if ((DBMode == "A") && (Articles->Find(ID, index)))
    UpdateUI(" " + ID + " already exists ");
  else {
    Art->Name = edName->Text;
    Art->Price = StrToFloat(edPrice->Text);
    Art->Stock = StrToInt(edStock->Text);
    if (DBMode == "A") {
      Articles->AddObject(ID, Art);
      DBMode = "B";
      UpdateUI(" Article was added ");
    }
    else {
      index = Grid->Row - 1;
      Articles->Sorted = false; // next line not allowed for sorted list
      Articles->Strings[index] = ID;
      Articles->Objects[index] = Art;
      Articles->Sorted = true;
      DBMode = "B";
      UpdateUI(" Article was modified ");
    }
    UpdateGrid();
  }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::btnCancelClick(TObject *Sender)
{
  DBMode = "B";
  UpdateUI(" No article added or modified ");
}
//---------------------------------------------------------------------------
void __fastcall TForm1::btnFindIDClick(TObject *Sender)
{
  int index;
  if (Articles->Find(edFind->Text, index)) {
    UpdateUI(" Article was found ");
    Grid->Row = index + 1;
    Grid->SetFocus();
  }
  else
    UpdateUI(" ID not found: " + edFind->Text + " ");
}
//---------------------------------------------------------------------------
void __fastcall TForm1::btnFindNameClick(TObject *Sender)
{
  int index, posit;
  int foundwhere = -1;
  String Name;
  String findwhat = UpperCase(edFind->Text); // Find case-insensitive
  for (index = 0; index < Articles->Count; index++)
  {
    TArticle *Art = static_cast<TArticle*>(Articles->Objects[index]);
    Name = UpperCase(Art->Name); // Find case-insensitive
    posit = Name.Pos(findwhat);
    if (posit > 0) {
      foundwhere = index;
      break;  // if found, break out of the loop
    }
  }
  if (foundwhere > -1) {
    UpdateUI(" Article was found ");
    Grid->Row = foundwhere + 1;
    Grid->SetFocus();
  }
  else
    UpdateUI(" NAME not found: " + edFind->Text + " ");
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormDestroy(TObject *Sender)
{
  delete Articles;
}
//---------------------------------------------------------------------------

